using System;
using System.CodeDom;
using System.CodeDom.Compiler;
using System.Reflection;
using System.Collections.Specialized;

namespace CodeDomDemo
{
	public class CCodeGenerator
	{
		CodeNamespace mynamespace;
		CodeTypeDeclaration myclass;
		CodeCompileUnit myassembly;

		static void Main(string[] args)
		{
			CCodeGenerator cg=new CCodeGenerator();
			cg.CreateNamespace();
			cg.CreateImports();
			cg.CreateClass();
			cg.CreateMember();
			cg.CreateProperty();
			cg.CreateMethod();
			cg.CreateEntryPoint();
			cg.SaveAssembly();
		}

		public void CreateNamespace()
		{
			mynamespace= new CodeNamespace();
			mynamespace.Name = "MyNamespace";
		}

		public void CreateImports()
		{
			mynamespace.Imports.Add(new CodeNamespaceImport("System"));
			mynamespace.Imports.Add(new CodeNamespaceImport("System.Drawing"));
			mynamespace.Imports.Add(new CodeNamespaceImport("System.Windows.Forms"));
		}

		public void CreateClass()
		{
			myclass = new CodeTypeDeclaration();
			myclass.Name = "CMyClass";
			myclass.IsClass = true;
			myclass.Attributes = MemberAttributes.Public;
			mynamespace.Types.Add(myclass);
		}

		public void CreateEntryPoint()
		{
			CodeEntryPointMethod mymain = new CodeEntryPointMethod();
			mymain.Name="Main";
			mymain.Attributes=MemberAttributes.Public | MemberAttributes.Static;
			CodeSnippetExpression snippet1=new CodeSnippetExpression("CMyClass x=new CMyClass()");
			CodeSnippetExpression snippet2=new CodeSnippetExpression("x.Message=\"Hello World\"");
			CodeSnippetExpression snippet3=new CodeSnippetExpression("Console.WriteLine(x.Message)");
			CodeSnippetExpression snippet4=new CodeSnippetExpression("Console.WriteLine(\"Answer: {0}\",x.AddNumbers(10,20))");
			CodeSnippetExpression snippet5=new CodeSnippetExpression("Console.ReadLine()");

			CodeExpressionStatement stmt1=new CodeExpressionStatement(snippet1);
			CodeExpressionStatement stmt2=new CodeExpressionStatement(snippet2);
			CodeExpressionStatement stmt3=new CodeExpressionStatement(snippet3);
			CodeExpressionStatement stmt4=new CodeExpressionStatement(snippet4);
			CodeExpressionStatement stmt5=new CodeExpressionStatement(snippet5);

            mymain.Statements.Add(stmt1);
			mymain.Statements.Add(stmt2);
			mymain.Statements.Add(stmt3);
			mymain.Statements.Add(stmt4);
			mymain.Statements.Add(stmt5);

			myclass.Members.Add(mymain);
		}

		public void CreateMember()
		{
			CodeMemberField mymemberfield=new CodeMemberField(typeof(System.String), "strMessage");
			myclass.Members.Add(mymemberfield);

		}

		public void CreateProperty()
		{
			CodeMemberProperty myproperty=new CodeMemberProperty();
			myproperty.Name="Message";
			myproperty.Type=new CodeTypeReference(typeof(string));
			//CodeParameterDeclarationExpression cpd1= new CodeParameterDeclarationExpression(typeof(int),"a");
			//myproperty.Parameters.Add(cpd1);
			
			myproperty.Attributes=MemberAttributes.Public;
			myclass.Members.Add(myproperty);

			CodeSnippetExpression getsnippet= new CodeSnippetExpression("return strMessage");
			CodeSnippetExpression setsnippet= new CodeSnippetExpression("strMessage=value");

			myproperty.GetStatements.Add(getsnippet);
			myproperty.SetStatements.Add(setsnippet);

		}

		public void CreateMethod()
		{
			CodeMemberMethod mymethod = new CodeMemberMethod();
			mymethod.Name="AddNumbers";
			CodeParameterDeclarationExpression cpd1= new CodeParameterDeclarationExpression(typeof(int),"a");
			CodeParameterDeclarationExpression cpd2= new CodeParameterDeclarationExpression(typeof(int),"b");
			mymethod.Parameters.Add(cpd1);
			mymethod.Parameters.Add(cpd2);
			CodeTypeReference ctr=new CodeTypeReference("System.Int32");
			mymethod.ReturnType=ctr;
			CodeSnippetExpression snippet1=new CodeSnippetExpression("System.Console.WriteLine(\"Adding :\" + a + \" And \" + b )");
			CodeSnippetExpression snippet2=new CodeSnippetExpression("return a+b");
			CodeExpressionStatement stmt1=new CodeExpressionStatement(snippet1);
			CodeExpressionStatement stmt2=new CodeExpressionStatement(snippet2);
			mymethod.Statements.Add(stmt1);
			mymethod.Statements.Add(stmt2);
			mymethod.Attributes=MemberAttributes.Public;
			myclass.Members.Add(mymethod);
		}


		public void SaveAssembly()
		{
			myassembly= new CodeCompileUnit();
			myassembly.Namespaces.Add(mynamespace);
			CompilerParameters compparams = new CompilerParameters(new string[]{"mscorlib.dll"});
			compparams.ReferencedAssemblies.Add("System.dll");
			compparams.ReferencedAssemblies.Add("System.Drawing.dll");
			compparams.ReferencedAssemblies.Add("System.Windows.Forms.dll");
			compparams.GenerateInMemory=false;
			compparams.GenerateExecutable=true;
			compparams.MainClass="MyNamespace.CMyClass";
			compparams.OutputAssembly=@"c:\bipin\HelloWorld.exe";
			Microsoft.CSharp.CSharpCodeProvider csharp = new Microsoft.CSharp.CSharpCodeProvider();
			ICodeCompiler cscompiler = csharp.CreateCompiler();
			CompilerResults compresult = cscompiler.CompileAssemblyFromDom(compparams,myassembly);
			if(compresult == null || compresult.Errors.Count>0)
			{
				for(int i=0;i<compresult.Errors.Count;i++)
				{
					Console.WriteLine(compresult.Errors[i]);
				}
				Console.ReadLine();
				Environment.Exit(1);
			}
		}

	}
}
